pub enum TypeAnnotation {
Show 30 variants
Int,
Float,
Bool,
String,
Byte,
Char,
Array(Box<TypeAnnotation>),
Map(Box<TypeAnnotation>, Box<TypeAnnotation>),
Set(Box<TypeAnnotation>),
CInt,
CFloat,
CBool,
CString,
CByte,
CChar,
CArray(Box<TypeAnnotation>),
CMap(Box<TypeAnnotation>, Box<TypeAnnotation>),
CSet(Box<TypeAnnotation>),
Fn,
Null,
Tuple(Rc<Vec<TypeAnnotation>>),
CTuple(Rc<Vec<TypeAnnotation>>),
Error,
CError,
Result(Box<TypeAnnotation>),
CResult(Box<TypeAnnotation>),
Record(String),
CRecord(String),
Enum(String),
CEnum(String),
}Expand description
The type of a variable, constant, or parameter binding.
Mutable variants (Int, Float, etc.) are produced by dec declarations.
Constant variants (CInt, CFloat, etc.) are produced by const declarations.
Array(T) / CArray(T) are the mutable / constant array forms.
Variants
Int
Mutable 64-bit signed integer.
Float
Mutable 64-bit float.
Bool
Mutable boolean.
String
Mutable string.
Byte
Mutable byte (u8).
Char
Mutable character.
Array(Box<TypeAnnotation>)
Mutable array with a typed element.
Map(Box<TypeAnnotation>, Box<TypeAnnotation>)
Set(Box<TypeAnnotation>)
CInt
Constant 64-bit signed integer.
CFloat
Constant 64-bit float.
CBool
Constant boolean.
CString
Constant string.
CByte
Constant byte.
CChar
Constant character.
CArray(Box<TypeAnnotation>)
Constant array with a typed element.
CMap(Box<TypeAnnotation>, Box<TypeAnnotation>)
CSet(Box<TypeAnnotation>)
Fn
A function value (used for fn-typed parameters and variables).
Null
Absence of a type - used as the default return type when none is annotated.
Tuple(Rc<Vec<TypeAnnotation>>)
CTuple(Rc<Vec<TypeAnnotation>>)
Error
CError
Result(Box<TypeAnnotation>)
CResult(Box<TypeAnnotation>)
Record(String)
A named record (struct) type, mutable binding.
CRecord(String)
A named record (struct) type, constant binding.
Enum(String)
A named tag (enum) type, mutable binding.
CEnum(String)
A named tag (enum) type, constant binding.